# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.forms.fields.extras.enable import EnableField
from project.forms.fields.extras.fqdn import FQDNField
from project.forms.fields.extras.landiskpass import LanDiskPassField
from project.forms.fields.extras.weekday import MultiWeekdayField
from project.forms.fields.extras.datetime import SelectTimeField
from project.forms.fields.extras.integer import IntegerField
from project.forms.fields.select import SelectField
from project.fal.clone import Clone


class DayField(SelectField):

    __choices = (
            ('nothing', _(u"LanDiskClone: day nothing")),
            ('exe', _(u"LanDIskClone: day execute")),
            ('exefull', _(u"LanDIskClone: day execute (full)")))

    def __init__(self, *args, **kwargs):
        kwargs['choices'] = self.__choices
        super(DayField, self).__init__(*args, **kwargs)

    @classmethod
    def conv(cls, val):
        self = cls()
        if isinstance(val, dict):  # ex) {"enable": True, "full": False}
            if val['enable']:
                if val['full']:
                    return self.__choices[2][0]
                else:
                    return self.__choices[1][0]
            else:
                return self.__choices[0][0]
        elif isinstance(val, (str, unicode)):  # choice value. ex) "nothing"
            if val == self.__choices[0][0]:
                return {'enable': False, 'full': False}
            elif val == self.__choices[1][0]:
                return {'enable': True, 'full': False}
            elif val == self.__choices[2][0]:
                return {'enable': True, 'full': True}
            else:
                return {'enable': False, 'full': False}
        return None


class Form(forms.Form):
    times = IntegerField(
            label=_(u'Times'),
            max_value=31,
            min_value=0,
            initial=0,
            required=True)
    schedule = EnableField(
            label=_(u'Schedule'))
    sun = DayField(label=_(u"Sun"))
    mon = DayField(label=_(u"Mon"))
    tue = DayField(label=_(u"Tue"))
    wed = DayField(label=_(u"Wed"))
    thu = DayField(label=_(u"Thu"))
    fri = DayField(label=_(u"Fri"))
    sat = DayField(label=_(u"Sat"))
    time = SelectTimeField(
            label=_(u'Time'),
            minute_step=5,
            show_seconds=False)

    def clean(self):
        data = self.cleaned_data
        if self.cleaned_data.get('schedule', None):
            for day in ['sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat']:
                ef = DayField.conv(self.cleaned_data.get(day, 'nothing'))
                if isinstance(ef, dict) and ef.get('enable'):
                    break
            else:
                raise ValidationError("%s: %s" % (
                        _(u'Day'),
                        _(u'This field is required.')))
            if self.cleaned_data.get('time', None) is None:
                raise ValidationError("%s: %s" % (
                        _(u'Time'),
                        _(u'This field is required.')))
        return data


class View(FuncView):

    views = ['apply', 'exe', 'exefull', 'stop']

    def __exists_generationlist(self):
        try:
            if len(Clone().query()['generationlist']):
                return True
            return False
        except:
            return False

    def __apply_config(self, clone, form):
            config = {
                'system': {
                    'times': form.cleaned_data['times'],
                    'schedule': {
                        'enable': form.cleaned_data['schedule'],
                        #'day': form.cleaned_data['day'],
                        'day': {
                            'sun': DayField.conv(form.cleaned_data['sun']),
                            'mon': DayField.conv(form.cleaned_data['mon']),
                            'tue': DayField.conv(form.cleaned_data['tue']),
                            'wed': DayField.conv(form.cleaned_data['wed']),
                            'thu': DayField.conv(form.cleaned_data['thu']),
                            'fri': DayField.conv(form.cleaned_data['fri']),
                            'sat': DayField.conv(form.cleaned_data['sat']),
                        },
                        'time': form.cleaned_data['time'],
                    },
                },
            }
            return clone.apply(config=config)


    def view(self, request):
        clone = Clone()
        config = clone.query()
        form = Form(initial={
            'times': config['system']['times'],
            'schedule': config['system']['schedule']['enable'],
            #'day': config['system']['schedule']['day'],
            'sun': DayField.conv(config['system']['schedule']['day']['sun']),
            'mon': DayField.conv(config['system']['schedule']['day']['mon']),
            'tue': DayField.conv(config['system']['schedule']['day']['tue']),
            'wed': DayField.conv(config['system']['schedule']['day']['wed']),
            'thu': DayField.conv(config['system']['schedule']['day']['thu']),
            'fri': DayField.conv(config['system']['schedule']['day']['fri']),
            'sat': DayField.conv(config['system']['schedule']['day']['sat']),
            'time': config['system']['schedule']['time'],
        })
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'exists_generationlist': self.__exists_generationlist(),
            },
        }

    def view_apply(self, request):
        with self.processing() as p:
            form = Form(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                clone = Clone()
                p.result(self.__apply_config(clone, form))
                self.set_message(clone.get_message())
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'exists_generationlist': self.__exists_generationlist(),
            },
        }

    def __view_exe(self, request, full=False):
        with self.processing() as p:
            form = Form(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                clone = Clone()
                if self.__apply_config(clone, form):
                    p.result(clone.backup(full=full, log=False))
                    self.set_message(clone.get_message())
                    if p.result():
                        self.set_message(_(u"LanDiskClone: Please wait until STATUS lump blinking stop."))
                else:
                    p.result(False)
                    self.set_message(clone.get_message())
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'exists_generationlist': self.__exists_generationlist(),
            },
        }

    def view_exe(self, request):
        return self.__view_exe(request, full=False)

    def view_exefull(self, request):
        return self.__view_exe(request, full=True)

    def view_stop(self, request):
        from project.fal.led import LED
        from project.fal.buzzer import Buzzer
        LED().status(True, blink=True, color='green')

        form = Form(data=request.POST)
        if form.is_valid():
            clone = Clone()
            result = clone.stop()
            self.set_message(clone.get_message())

        LED().status(True, blink=False, color='green')
        if result:
            Buzzer().ok()
        else:
            Buzzer().error()
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'exists_generationlist': self.__exists_generationlist(),
            },
        }
